//white tracking function, scans source image for the greatest value of brightness in vertical columns
//the return value is the brightest column scanned
static int trackWhite(PImage source, int columns, int columnWidth, int speed){
  int [] brightnessTotal = new int[columns];
  for(int i = 0; i < columns; i++){
    for(int j = 0; j < source.height; j++){
      for(int k = 0; k < columnWidth; k += speed){
        int pixel = (i * columnWidth) + k + j * source.width;
        brightnessTotal[i] += grey(source.pixels[pixel]);
      }
    }
  }
  int brightest = -1;
  int bright = -1;
  for(int i = 0; i < columns; i++){
    if(brightnessTotal[i] > brightest){
      bright = i;
      brightest = brightnessTotal[i];
    }
  }
  return bright;
}

//integer brightness function for extra speed
static int grey(color p){
  return max((p >> 16) & 0xFF, (p >> 8) & 0xFF, p & 0xFF);
}

//function to create a table of colour values for TetrisField elements
void initBlockColor(int frames){
  int [] block = new color[8];
  block[0] = color(255);
  block[1] = color(255, 255, 0);
  block[2] = color(0, 255, 255);
  block[3] = color(255, 0, 255);
  block[4] = color(0, 0, 255);
  block[5] = color(255, 125, 0);
  block[6] = color(0, 255, 0);
  block[7] = color(255, 0, 0);
  blockColor = new color[block.length * frames];
  blockColorEdge = new color[block.length * frames];
  for(int i = 0; i < frames; i++){
    for(int j = 1; j < block.length; j++){
      blockColor[j + i * block.length] = color(constrain(red(block[j]) + ((260.0 / frames) * i), 0, 255), constrain(green(block[j]) + ((260.0 / frames) * i), 0, 255) , constrain(blue(block[j]) + ((260.0 / frames) * i), 0, 255));
      blockColorEdge[j + i * block.length] = color(constrain(red(block[j]) + ((150.0 / frames) * i), 0, 150), constrain(green(block[j]) + ((150.0 / frames) * i), 0, 150) , constrain(blue(block[j]) + ((150.0 / frames) * i), 0, 150));
    }
  }
  blockColor[blockColor.length - 1] = color(255);
  blockColorEdge[blockColorEdge.length - 1] = color(150);
}

//function to move block according to trackWhite function's report
void moveToVideo(){
  if(block.x < (locale - 1)){
    block.move(1);
  }
  if(block.x > (locale - 1)){
    block.move(-1);
  }
}

//function to rotate blocks based on input from microphone and also display fft data
void loudNoise(){
  float getLevel = LiveInput.getLevel();//returns a value from 0 to 0.5
  if(showMicrophone){
    noStroke();
    fill(0, 255, 0);
    rect(0, 0, width * (getLevel * 2), 20);
    fill(255, 0, 0);
    rect(-5 + width * (rotateThreshold * 2), 0, 10, 20);
  }
  if(getLevel >= rotateThreshold && !microphoneLock){
    microphoneLock = true;
    block.rotate();
  }
  if(getLevel < rotateThreshold && microphoneLock){
    microphoneLock = false;
  }
  //FFT display
  stroke(0, 255, 0);
  LiveInput.getSpectrum();
   for ( int i = 0; i < LiveInput.spectrum.length; i++){ 
      line(i*2, height, i*2, height - LiveInput.spectrum[i]/10); 
   } 
}

public void stop(){ 
  Sonia.stop(); 
  super.stop(); 
}
