void setup(){
  size(1024, 768, OPENGL);
  smooth();
  framerate(24);
  and = new Character('0');
  field = new TetrisField((width / 2) - ((height / fieldHigh) * (fieldWide / 2)), 0, fieldWide, fieldHigh, height / fieldHigh, height / fieldHigh, dropRate, increaseSpeed);
  nextBlock = new TetrisNextBlock();
  block = new TetrisBlock();
  building = new Building();
  Sonia.start(this);
  LiveInput.start(128);
  initBlockColor(squareFadeRate);
  ellipseMode(CENTER);
  if(exhibitMode){
    capture = new Capture(this, captureDeviceName, captureWide, captureHigh, 12);
  }
  architecture = new PImage[imagesToLoad + 1];
  for(int i = 1; i < imagesToLoad + 1; i++){
    architecture[i] = new PImage(field.sizex, field.sizey);
    PImage temp = loadImage("window" + i + ".jpg");
    architecture[i].copy(temp, 0, 0, temp.width, temp.height, 0, 0, architecture[i].width, architecture[i].height);
    architecture[i].loadPixels();
  }
  mugShot = new PImage[mugsToLoad][];
  for(int i = 0; i < mugsToLoad; i++){
    mugShot[i] = new PImage[mugShotTable[i]];
    for(int j = 0; j < mugShotTable[i]; j++){
      mugShot[i][j] = new PImage(100, 150);
      PImage temp = loadImage("mug" + i + "-" + j + ".jpg");
      mugShot[i][j].copy(temp, 0, 0, temp.width, temp.height, 0, 0, mugShot[i][j].width, mugShot[i][j].height);
      mugShot[i][j].loadPixels();
    }
  }
  quadTextSize(titleSize);
  String s0 = "architectris";
  titleText = new QuadTextWobble(s0, (width / 2) - (quadTextWidth(s0) / 2), height / 2, wobbleRange, wobbleSpeed);
  int rightOfField = field.x + (field.sizex * field.wide);
  highScoreText = new QuadTextWobble("high score", rightOfField + 10, field.y + 40, wobbleRange, wobbleSpeed);
  scoreText = new QuadTextWobble("score", rightOfField + 10, field.y + 80, wobbleRange, wobbleSpeed);
  linesText = new QuadTextWobble("lines", rightOfField + 10, field.y + 120, wobbleRange, wobbleSpeed);
  nextPieceText = new QuadTextWobble("next piece", rightOfField + 10, field.y + 160, wobbleRange, wobbleSpeed);
  rankText = new QuadTextWobble("rank", rightOfField + 10, field.y + 360, wobbleRange, wobbleSpeed);
  rankingMugShot = int(random(mugShotTable[0]));
}

void draw(){
  if(!exhibitMode){
    //sets movement of block to mouse movement
    background(0);
    fill(255);
    ellipse(mouseX, mouseY, 50, 50);
    loadPixels();
    locale = trackWhite(g, field.wide, trackColumnWidthRect, trackSpeed);
  }
  if(showCapture){
    capture.updatePixels();
    image(capture, 0, 0, width, height);
    //rectangle depicting locale of tracker on webcam image
    noFill();
    stroke(200);
    rect((locale) * trackColumnWidthRect, 0, trackColumnWidthRect, height);
    //rectangle depicting locale of tracker on TetrisField
    noStroke();
    fill(150, 150, 150, 150);
    rect(field.x + ((locale) * field.sizex), 0, field.sizex, field.y + field.high * field.sizey);
  } 
  else {
    background(0);
  }
  switch(gameState){
  case title:
    if(building.square != null){
      building = new Building();
    }
    stroke(textStroke);
    noFill();
    titleText.draw();
    break;
  case playGame:
    if(block.initNextBlock){
      block.initNextBlock = false;
      nextBlock = new TetrisNextBlock();
    }
    if(block.checkRows){
      field.checkRows();
      block = new TetrisBlock();
    }
    block.draw();
    field.draw();
    building.draw(30,30);
    moveToVideo();
    loudNoise();
    int rightOfField = field.x + (field.sizex * field.wide);
    stroke(textStroke);
    noFill();
    highScoreText.draw();
    scoreText.draw();
    linesText.draw();
    nextPieceText.draw();
    rankText.draw();
    quadText("            " + bestScore, rightOfField + 10, field.y + 40);
    quadText("       " + field.score, rightOfField + 10, field.y + 80);
    quadText("       " + field.lines, rightOfField + 10, field.y + 120);
    quadText("           ", rightOfField + 10, field.y + 160);
    String [] string = split(rankName[ranking]);
    for(int i = 0; i < string.length; i++){
      if(i == 0){
        quadText("      " + string[i], rightOfField + 10, field.y + 360);
      }
      quadText("      " + string[i], rightOfField + 10, field.y + 360 + (i * 30));
    }
    nextBlock.draw(rightOfField - 75, field.y + 200);
    mugShot[ranking][rankingMugShot].updatePixels();
    image(mugShot[ranking][rankingMugShot], rightOfField + 100, 450);
    break;
  case gameOver:
    stroke(textStroke);
    noFill();
    youScoredText.draw();
    andMadeText.draw();
    yourRankText.draw();
    break;
  }
}

void captureEvent(Capture capture) { 
  if(exhibitMode){
  capture.read();
  locale = trackWhite(capture, field.wide, trackColumnWidth, trackSpeed);
  //flip tracking to simulate effect of looking into a mirror
  locale = field.wide - 1 - locale;
  }
}

void keyPressed(){
  if(exhibitMode){
    switch(gameState){
    case title:
      gameState = playGame;
      quadTextSize(playGameSize);
      break;
    case playGame:
      while(!block.checkRows){
        block.drop(true);
      }
      break;
    case gameOver:
      gameState = title;
      quadTextSize(titleSize);
      break;
    }
  }
  else{
    switch(keyCode){
    case UP:
      block.rotate();
      break;
    case DOWN:
      while(!block.checkRows){
        block.drop(true);
      }
      break;
    case ENTER:
      switch(gameState){
      case title:
        gameState = playGame;
        quadTextSize(playGameSize);
        break;
      case gameOver:
        gameState = title;
        quadTextSize(titleSize);
        break;
      }
    }
    switch(key){
    case '=':
    case '+':
      rotateThreshold = constrain(rotateThreshold + 0.05, 0.0, 0.5);
      break;
    case '-':
    case '_':
      rotateThreshold = constrain(rotateThreshold - 0.05, 0.0, 0.5);
      break;
    }
  }
}
